unit MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, System.IOUtils,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  Bcl.Lang,
  Sphinx.Comp.Config, Sparkle.HttpServer.Module, Sparkle.HttpServer.Context, Sphinx.Server.Module, Aurelius.Drivers.Interfaces,
  Aurelius.Drivers.SQLite, Aurelius.Engine.DatabaseManager, Aurelius.Comp.DBSchema, Aurelius.Comp.Connection,
  XData.Comp.ConnectionPool,
  Sparkle.Comp.Server, Sparkle.Comp.HttpSysDispatcher, XData.Comp.Server, Sphinx.Comp.Server,
  Sparkle.Module.Static, Sparkle.Comp.StaticServer, Sparkle.Comp.CorsMiddleware,

  IndySecureMailClient, Aurelius.Sql.Sqlite, Aurelius.Schema.Sqlite,
  Aurelius.Drivers.UniDac, Data.DB, DBAccess, Uni, UniProvider,
  SQLiteUniProvider;

type
  TServerForm = class(TForm)
    SphinxServer1: TSphinxServer;
    Dispatcher: TSparkleHttpSysDispatcher;
    XDataConnectionPool1: TXDataConnectionPool;
    AureliusConnection1: TAureliusConnection;
    Memo1: TMemo;
    AureliusDBSchema1: TAureliusDBSchema;
    SphinxConfig1: TSphinxConfig;
    Button1: TButton;
    WebClientDispatcher: TSparkleHttpSysDispatcher;
    UniConnection1: TUniConnection;
    SQLiteUniProvider1: TSQLiteUniProvider;
    SparkleStaticServer1: TSparkleStaticServer;
    SphinxServer1CORS: TSparkleCorsMiddleware;
    procedure FormCreate(Sender: TObject);
    procedure SphinxConfig1GetSigningData(Sender: TObject; Args: TGetSigningDataArgs);
    procedure SphinxConfig1GeneratePasswordResetToken(Sender: TObject; Args: TGeneratePasswordResetTokenArgs);
    procedure SphinxConfig1GenerateEmailConfirmationToken(Sender: TObject; Args: TGenerateEmailConfirmationTokenArgs);
    procedure Button1Click(Sender: TObject);
    procedure SphinxConfig1ConfigureToken(Sender: TObject; Args: TConfigureTokenArgs);
    procedure DispatcherStart(Sender: TObject);
  strict private
    procedure ConfigureClients;
    procedure Log(const Msg: string);
  end;

var
  ServerForm: TServerForm;

implementation

uses
  ServerUtils;
{$R *.dfm}

procedure TServerForm.Button1Click(Sender: TObject);
begin
  // Set folder location for web client HTML files and start serving those pages
  SparkleStaticServer1.RootDir := TPath.Combine(TPath.GetDirectoryName(ParamStr(0)), 'WebClientFiles');
  WebClientDispatcher.Start;
  log(SparkleStaticServer1.RootDir);
end;

procedure TServerForm.ConfigureClients;
var
  Client: TSphinxClientApp;
begin
  // Create web client
  Client := SphinxConfig1.Clients.Add;
  Client.ClientId := 'web';
  Client.DisplayName := 'Projekt-Management';
  Client.RedirectUris.Add('http://178.254.40.246:2001/pm/webclient/');
  Client.RequireClientSecret := False;
  Client.AllowedGrantTypes := [TGrantType.gtAuthorizationCode];
  Client.ValidScopes.Add('openid');
  Client.ValidScopes.Add('email');
end;

procedure TServerForm.DispatcherStart(Sender: TObject);
begin
  Log('Dispatcher started');
end;

procedure TServerForm.FormCreate(Sender: TObject);
begin
  // Initialize the language engine
  TLang.Init;

  // Set the SQLite database location (same file name as .exe but with .db extension)
  AureliusConnection1.Params.Values['Database'] := ChangeFileExt(ParamStr(0), '.db');

  // Create all needed tables and fields in database
  AureliusDBSchema1.UpdateDatabase;

  // Manually create some users
  SeedUsers(SphinxServer1.CreateContext);

  // Start the server
  ConfigureClients;
  Dispatcher.Start;
  Log('Server started at ' + StringReplace(SphinxServer1.BaseUrl, '+', 'localhost', []));
  Log('Watch this space for email confirmation and password reset tokens.');
end;

procedure TServerForm.Log(const Msg: string);
begin
  TThread.Queue(nil, procedure
    begin
      Memo1.Lines.Add(Msg);
    end);
end;

procedure TServerForm.SphinxConfig1ConfigureToken(Sender: TObject; Args: TConfigureTokenArgs);
var
  TenantId: string;
  P: Integer;
begin
  TenantId := Args.User.Email.ValueOrDefault;
  P := Pos('@tmssoftware.local', TenantId);
  if P > 1 then
    TenantId := UpperCase(Copy(TenantId, 1, 1)) + Copy(TenantId, 2, P - 2);
  Args.Token.Claims.AddOrSet('tenantId', TenantId)
end;

procedure TServerForm.SphinxConfig1GenerateEmailConfirmationToken(Sender: TObject; Args: TGenerateEmailConfirmationTokenArgs);
var
  Mail: TSSLEmail;
begin
  Log('Send Email confirmation token.');

  // Send Token as Mail to Operator
  Mail := TSSLEmail.Create('smtp.1blu.de',
                           465,
                           'o142289_0-info', 'uP6fxXa5l&');

  try
    Mail.edSenderName := 'Andreas Stapel';
    Mail.edSenderEmail := 'info@splan-concept.de';
    Mail.edToEmail := Args.User.Email;
    Mail.edSubject := 'Passwort fr das Projektmanagement';
    Mail.edBody.Text := 'Das Passwort fr die erste Anmeldung: '+ Args.Token;

    Mail.SendEmail;
  finally
    Mail.Free;
    Log('Email confirmation token sent to: '+args.User.Email);
  end;
end;

procedure TServerForm.SphinxConfig1GeneratePasswordResetToken(Sender: TObject; Args: TGeneratePasswordResetTokenArgs);
var
  Mail: TSSLEmail;
begin
  Log('Send Password reset token');

  // Send Token as Mail to Operator
  Mail := TSSLEmail.Create('smtp.1blu.de',
                           465,
                           'o142289_0-info', 'uP6fxXa5l&');

  try
    Mail.edSenderName := 'Andreas Stapel';
    Mail.edSenderEmail := 'info@splan-concept.de';
    Mail.edToEmail := Args.User.Email;
    Mail.edSubject := 'Passwort zurcksetzen';
    Mail.edBody.Text := 'Code zum zurcksetzen Ihres Passworts: '+ Args.Token;

    Mail.SendEmail;
  finally
    Mail.Free;
    Log('Password reset token sent to: '+args.User.Email);
  end;
end;

procedure TServerForm.SphinxConfig1GetSigningData(Sender: TObject; Args: TGetSigningDataArgs);
begin
  Args.Data.Key := TEncoding.UTF8.GetBytes('a secret that has the minimum length');
end;

end.
